#!/usr/bin/python

"""
Art: A module for loading a game's art all at once.
"""

__author__ = "Kris Schnee"
__date__ = "2008.1.24"
__license__ = "Public Domain"

import os
import pygame
pygame.init()

READABLE_GRAPHICS_FORMATS = ["bmp","png","gif","jpg"]
GRAPHICS_DIRECTORY = "graphics"
TILE_DIRECTORY = os.path.join(GRAPHICS_DIRECTORY,"tiles")
PORTRAIT_DIRECTORY = os.path.join(GRAPHICS_DIRECTORY,"portraits")
FONT_DIRECTORY = os.path.join(GRAPHICS_DIRECTORY,"fonts")
POSTER_DIRECTORY = os.path.join(GRAPHICS_DIRECTORY,"posters")
SPRITE_DIRECTORY = os.path.join(GRAPHICS_DIRECTORY,"sprites")

DEFAULT_FONT = None
TILE_SIZE = 64

## Sprite frame data.
SPRITE_SIZE = [96,128] ## Animation frame size
DEFAULT_SPRITE_FRAMES_PER_ROW = 3
DEFAULT_SPRITE_HOW_MANY_ROWS = 4
DEFAULT_SPRITE_STARTING_FRAME = 7
SPRITE_COLORKEY = (32,156,0,255)

## Always create a display surface so that art can load, if none exists yet.
try:
    screen = screen
except:
    screen = pygame.display.set_mode((1,1))

"""Holds the game's art.

Covers several categories:
-Posters: Backgrounds, misc. illustrations
-Portraits: Character faces
-Icons: Mini portraits
-Tiles: small images used to draw a landscape
-Sprites: objects with animation frames and drawing functions
-Fonts: Text designs.

Rather than load the same art repeatedly, this class holds all art
that's available to the program in subdirectories of \graphics.
The art is then available to other parts of the program.
"""

fonts = {}
portraits = {}
icons = {}
posters = {}
tiles = {}
sprite_sheets = {}

def LoadGraphics():
    try:
        if DEFAULT_FONT:
            fonts["default"] = pygame.font.Font(os.path.join(FONT_DIRECTORY,DEFAULT_FONT),32)
        else:
            try:
                fonts["default"] = pygame.font.Font(None,32)
            except:
                pass ## Fails in EXE form in some cases.
        font_filenames = [f for f in os.listdir(FONT_DIRECTORY) if f[-3:] == "ttf"]
        for font_name in font_filenames:
            fonts[font_name[:-4]] = pygame.font.Font(os.path.join(FONT_DIRECTORY,font_name),32)
    except:
        pass

    try:
        image_filenames = [f for f in os.listdir(PORTRAIT_DIRECTORY) if f[-3:] in READABLE_GRAPHICS_FORMATS]
        for image_name in image_filenames:
            surface = pygame.image.load(os.path.join(PORTRAIT_DIRECTORY,image_name)).convert()
            surface.set_colorkey(SPRITE_COLORKEY)
            portraits[image_name[:-4]] = surface
            icons[image_name[:-4]] = pygame.transform.scale(surface,(TILE_SIZE,TILE_SIZE))
    except:
        pass

    try:
        image_filenames = [f for f in os.listdir(POSTER_DIRECTORY) if f[-3:] in READABLE_GRAPHICS_FORMATS]
        for image_name in image_filenames:
            surface = pygame.image.load(os.path.join(POSTER_DIRECTORY,image_name)).convert_alpha()
            posters[image_name[:-4]] = surface
    except:
        pass

    try:
        image_filenames = [f for f in os.listdir(TILE_DIRECTORY) if f[-3:] in READABLE_GRAPHICS_FORMATS]
        for image_name in image_filenames:
            surface = pygame.image.load(os.path.join(TILE_DIRECTORY,image_name))
            tiles[image_name[:-4]] = surface
    except:
        pass

    try:
        image_filenames = [f for f in os.listdir(SPRITE_DIRECTORY) if f[-3:] in READABLE_GRAPHICS_FORMATS]
        for image_name in image_filenames:
            surface = pygame.image.load(os.path.join(SPRITE_DIRECTORY,image_name)).convert()
            surface.set_colorkey(SPRITE_COLORKEY)
            sprite_sheets[image_name[:-4]] = surface
    except:
        pass

def Inventory():
    text = ""
    """Show all available files."""
    text += "Portraits:"
    for key in portraits.keys():
        text += "\n  -"+key
    text += "\nTiles:"
    for key in tiles.keys():
        text += "\n  -"+key
    text += "\nPosters:"
    for key in posters.keys():
        text += "\n  -"+key
    text += "\nFonts:"
    for key in fonts.keys():
        text += "\n  -"+key
    text += "\nSprite sheets:"
    for key in sprite_sheets.keys():
        text += "\n  -"+key
    return text

def Write(text,font="default",color=(255,255,255)):
    """Return rendered text; shortcut for pygame.font.render."""
    try:
        return fonts[font].render(text,1,color)
    except:
        raise "Error: Font '"+font+"' not found."


## Autorun
LoadGraphics()
