#!/usr/bin/python
"""
Match Game
by Kris Schnee.

A playable piece of a puzzle game, using the basic game mechanic
of the game "Bejeweled" and numerous other puzzle games:
match 3+ objects of a kind in a row or column to make them vanish.
"""

##___________________________________
##          ~Header~
##___________________________________

__author__ = "Kris Schnee"
__version__ = "2008.2.18"
__license__ = "Public Domain"


##___________________________________
##      ~Imported Modules~
##___________________________________

## Standard
import random
import os

## Third-party
import pygame
from pygame.locals import *

## Mine
import ringtale
import art
import driftwood
import conch

##___________________________________
##            ~Constants~
##___________________________________

TILE_KINDS = ["rrggbbc","rrggbbpc","rrggbbyypc"]
TILE_SIZE = 64

TILES = {" ":"empty","p":"pc_gem_pink","r":"pc_gem_red","g":"pc_gem_green","b":"pc_gem_blue","c":"pc_chest","y":"pc_gem_yellow"}

SCORES = [150,250]
BACKGROUNDS = [art.posters["bg_coins"],art.posters["bg_raccoon"],art.posters["bg_gold"]]

art.fonts["default"] = art.fonts["EagerNaturalist"]
from conch import MUSIC_END_EVENT ## Trigger an event when music ends.

##___________________________________
##            ~Classes~
##___________________________________

##random.seed("Aqua Stream")

class Tile:
    def __init__(self,coords,kind):
        self.coords = coords
        self.screen_coords = [self.coords[0]*TILE_SIZE,self.coords[1]*TILE_SIZE]
        self.kind = kind
        self.image = art.tiles[TILES[self.kind]]

    def SetCoords(self,coords):
        self.coords = coords
        self.screen_coords = [self.coords[0]*TILE_SIZE,self.coords[1]*TILE_SIZE]




class MatchGameBoard:
    def __init__(self):
        self.board_size = (8,8)
        self.level = 0
        self.grid = []
        for y in range(self.board_size[1]):
            for x in range(self.board_size[0]):
                kind = self.RandomTileKind()
                self.grid.append(Tile((x,y),kind))

    def RandomTileKind(self):
        kinds = TILE_KINDS[self.level]
        kind = kinds[random.randint(0,len(kinds)-1)]
        return kind

    def __repr__(self):
        text = ""
        for y in range(self.board_size[1]):
            for x in range(self.board_size[0]):
                text += self.grid[y*8+x].kind
            text += "\n"
        return text

    def FillTop(self):
        for x in range(self.board_size[0]):
            tile_here = self.grid[x]
            if tile_here.kind == " ":
                self.grid[x] = Tile(coords=(x,0),kind=self.RandomTileKind())

    def FillAll(self):
        matches = True
        while matches or (" " in [t.kind for t in self.grid]):
            if matches and matches is not True:
                self.ClearTiles(matches)
            self.FillTop()
            self.MakeTilesFall()
            matches = self.FindMatches()

    def FindMatches(self):
        """Follow the standard match-game rule."""
        matches = []
        ## Horizontal matches
        for y in range(self.board_size[1]):
            start_x = 0
            current_kind = None
            for x in range(self.board_size[0]+1):
                if x == self.board_size[1]:
                    this_kind = " "
                else:
                    this_kind = self.grid[y*self.board_size[1]+x].kind
                if this_kind == current_kind:
                    pass
                else:
                    ## Chain ended. How long is it?
                    length = x - start_x
                    if length >= 3 and current_kind != " ":
                        matches.append({"kind":current_kind,"length":length,"axis":"x","x":(start_x,x-1),"y":y})
                    start_x = x
                    current_kind = this_kind

        ## Vertical matches
        for x in range(self.board_size[0]):
            start_y = 0
            current_kind = None
            for y in range(self.board_size[1]+1):
                if y == self.board_size[1]:
                    this_kind = " "
                else:
                    this_kind = self.grid[y*self.board_size[1]+x].kind
                if this_kind == current_kind:
                    pass
                else:
                    ## Chain ended. How long is it?
                    length = y - start_y
                    if length >= 3 and current_kind != " ":
                        matches.append({"kind":current_kind,"length":length,"axis":"y","x":x,"y":(start_y,y-1)})
                    start_y = y
                    current_kind = this_kind

        return matches

    def ClearTiles(self,sets_cleared):
        for s in sets_cleared:
            if s["axis"] == "x":
                for x in range(s["x"][0],s["x"][1]+1):
                    self.grid[s["y"]*self.board_size[0]+x] = Tile(coords=(x,s["y"]),kind=" ")
            elif s["axis"] == "y":
                for y in range(s["y"][0],s["y"][1]+1):
                    self.grid[y*self.board_size[0]+s["x"]] = Tile(coords=(s["x"],y),kind=" ")

    def MakeTilesFall(self):
        for n in range(self.board_size[1]):
            for y in range(self.board_size[1]-1):
                for x in range(self.board_size[0]):
                    below = self.grid[(y+1)*self.board_size[0]+x]
                    if below.kind == " ":
                        this_tile = self.grid[(y)*self.board_size[0]+x]
                        self.grid[(y+1)*self.board_size[0]+x] = this_tile
                        self.grid[y*self.board_size[0]+x] = below
                        this_tile.SetCoords((x,y+1))
                        below.SetCoords((x,y))

    def SwitchTiles(self,c1,c2):
        tile_1 = self.grid[c1[1]*self.board_size[0]+c1[0]]
        tile_2 = self.grid[c2[1]*self.board_size[0]+c2[0]]
        self.grid[c1[1]*self.board_size[0]+c1[0]] = tile_2
        self.grid[c2[1]*self.board_size[0]+c2[0]] = tile_1
        tile_2.SetCoords(c1)
        tile_1.SetCoords(c2)



class Game( ringtale.Framework ):
    def __init__(self):
        ringtale.Framework.__init__(self)
        conch.LoadAllSounds()
        self.board = MatchGameBoard()

        self.selected_tile = None

        self.current_background = BACKGROUNDS[self.board.level]

        self.tile_size = 64

        for tile in art.tiles.values():
            tile.set_colorkey((255,0,255))            

        self.PushState("Main")

    def MainSetup(self):
        self.interface = []
        self.notes["score"] = 0
        self.notes["max_chain"] = 0
        self.notes["switch_range"] = 1
        self.notes["refills"] = 3
        self.notes["wLevel"] = driftwood.Widget(name="wLevel",coords=(512,0,288,50),text="Level "+str(self.board.level+1))
        self.interface.append(self.notes["wLevel"])
        self.notes["wScore"] = driftwood.Widget(name="wScore",coords=(512,50,288,50),text=str(self.notes["score"])+" Points")
        self.interface.append(self.notes["wScore"])
        self.notes["wMaxChain"] = driftwood.Widget(name="wMaxChain",coords=(512,100,288,50),text="Max Chain: "+str(self.notes["max_chain"]))
        self.interface.append(self.notes["wMaxChain"])
        self.notes["wRefills"] = driftwood.Widget(name="wRefills",coords=(512,150,288,50),text="Refills: "+str(self.notes["refills"]))
        self.interface.append(self.notes["wRefills"])
        self.board.FillAll()

    def MainLogic(self):
        for event in pygame.event.get():
            if event.type == QUIT:
                self.PopState()
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    self.PopState()
                elif event.key == K_SPACE:
                    if self.notes["refills"] > 0:
                        self.notes["refills"] -= 1
                        self.notes["wRefills"].SetText("Refills: "+str(self.notes["refills"]))
                        self.board.FillAll()

            elif event.type == MOUSEBUTTONDOWN:
                tile_clicked = self.GetTileCoordsClicked(event.pos)
                if tile_clicked:
                    if self.selected_tile:
                        ## Is this a valid switch position?
                        d = OrthoDistance(self.selected_tile,tile_clicked)
                        if d <= self.notes["switch_range"]:
                            conch.PlaySound("ok")
                            self.board.SwitchTiles(self.selected_tile,tile_clicked)
                            self.selected_tile = None
                        else:
                            conch.PlaySound("no")
                    else:
                        self.selected_tile = tile_clicked

            elif event.type == MUSIC_END_EVENT:
                conch.PlaySong()

        matches = self.board.FindMatches()
        if matches:
            chain = 1
            score_multiplier = 1
            for match in matches:
                conch.PlaySound("score")
                self.notes["score"] += (match["length"] * match["length"]) * score_multiplier
                score_multiplier += 1
                chain += 1
            self.notes["wScore"].SetText(str(self.notes["score"])+" Points")
            if chain > self.notes["max_chain"]:
                self.notes["max_chain"] = chain
                self.notes["wMaxChain"].SetText("Max Chain: "+str(self.notes["max_chain"]))

            if self.board.level < len(SCORES):
                score_to_next_level = SCORES[self.board.level]
                if self.notes["score"] >= score_to_next_level:
                    self.board.level += 1
                    conch.PlaySound("levelup")
                    self.current_background = BACKGROUNDS[self.board.level]
                    if self.board.level == len(SCORES):
                        self.notes["wLevel"].SetText("Level MAX")
                    else:
                        self.notes["wLevel"].SetText("Level: "+str(self.board.level+1))

            self.board.ClearTiles(matches)

        self.board.MakeTilesFall()

    def MainDraw(self):
        self.DrawBoard()
        if self.selected_tile:
            x, y = self.selected_tile
            pygame.draw.rect(screen,(0,255,255),(x*64,y*64,64,64),3)
        for i in self.interface:
            i.Draw()

    def GetTileCoordsClicked(self,screen_coords):
        absolute_x = screen_coords[0]
        absolute_y = screen_coords[1]
        tile = (absolute_x/self.tile_size,absolute_y/self.tile_size)
        if tile[0] >= 0 and tile[0] < self.board.board_size[0] and tile[1] >= 0 and tile[1] < self.board.board_size[1]:
            return tile
        else:
            return None ## Impossible location.

    def DrawBoard(self):
        screen.blit(self.current_background,(0,0))    
        for y in range(self.board.board_size[1]):
            for x in range(self.board.board_size[0]):
                tile = self.board.grid[y*self.board.board_size[0]+x]
                kind = tile.kind
                sx, sy = tile.screen_coords
                screen.blit(tile.image,(sx,sy))

        pygame.display.update()


##___________________________________
##            ~Functions~
##___________________________________

def OrthoDistance(c1,c2):
    return abs(c1[0]-c2[0]) + abs(c1[1]-c2[1])

##__________________________________
##            ~Autorun~
##__________________________________

if __name__ == "__main__":

    screen = pygame.display.set_mode((800,512))
    pygame.display.set_caption("Match Game")

    conch.LoadSong("03 - New Dawn.ogg","main")
    conch.PlaySong("main")

    g = Game()
    g.MainLoop()

    conch.StopMusic()

